# Base module used by Aerosol color classes.
module AS::Color
	# Apply a block to each pair of components.
  def apply_rgba(other)
    ar, ag, ab, aa = self.to_rgba
    br, bg, bb, ba = color.to_rgba
    
    r = yield(ar, br)
    g = yield(ag, bg)
    b = yield(ab, bb)
    a = yield(aa, ba)
    
    return AS::Color::RGBA.new(r, g, b, a)
  end
  
  # Returns the RGB color components.
  def to_rgb
    to_rgba[0, 3]
  end
  
  # Linear interpolation between two colors.
  def lerp(t, color)
    apply_rgba(color){|a, b| (a - b)*t + b}
  end
  
  def +(color)
    apply_rgba(color){|a, b| a + b}
  end
  
  def -(color)
    apply_rgba(color){|a, b| a - b}
  end
  
  def *(color)
    apply_rgba(color){|a, b| a*b}
  end
  
  def inverse_rgb
    r, g, b, a = to_rgba
    AS::Color.new(1 - r, 1 - g, 1 - b, a)
  end
  
  def inverse_rgba
    r, g, b, a = to_rgba
    AS::Color.new(1 - r, 1 - g, 1 - b, 1 - a)
  end
  
  # Returns a AS::PackedColor for this color.
  def pack
    AS::PackedColor.new(self)
  end

	# RGB AS::Color class.
  class RGB
    include AS::Color
  
    attr_reader :r, :g, :b, :a
  
    def initialize(r, g, b, a=1)
      @r = r.to_f.clamp(0, 1)
      @g = g.to_f.clamp(0, 1)
      @b = b.to_f.clamp(0, 1)
      @a = a.to_f.clamp(0, 1)
    end
    
    
    def to_rgba
      [@r, @g, @b, @a]
    end
  end
  
  # HSV AS::Color class
  class HSV
    include AS::Color
  
    attr_reader :h, :s, :v, :a
  
	# All values should lie in the range[0, 1]
    def initialize(h, s, v, a=1)
      @h = h%1
      @s = s.to_f.clamp(0, 1)
      @v = v.to_f.clamp(0, 1)
      @a = a.to_f.clamp(0, 1)
    end
    
    def to_rgba
      return RGB.new(v, v, v, a) if s == 0
      
      h, f = (self.h*6).divmod(1)
      
      p = v*(1 - s)
      q = v*(1 - s*f)
      t = v*(1 - s*(1 - f))
      
      case h.to_i
      when 0: [v, t, p, a]
      when 1: [q, v, p, a]
      when 2: [p, v, t, a]
      when 3: [p, q, v, a]
      when 4: [t, p, v, a]
      else [v, p, q, a]
      end
    end
  end
  
  Black = RGB.new(0, 0, 0)
  White = RGB.new(1, 1, 1)
  
  Red = RGB.new(1, 0, 0)
  Green = RGB.new(0, 1, 0)
  Blue = RGB.new(0, 0, 1)
  
  Yellow = RGB.new(1, 1, 0)
  Magenta = RGB.new(1, 0, 1)
  Cyan = RGB.new(0, 1, 1)
end

# A packed representation of an AS::Color used by AS::Scene::ColorNode
class AS::PackedColor
  attr_reader :color, :packed_color

  def initialize(color=AS::Color::Red)
    @packed_color = ""
		self.color = color
  end
  
  def color=(color)
    @color = color
		@packed_color.replace(color.to_rgba.pack('f*'))
  end
end