# Gamestates are smart gameloops.  They not only handle the game loop
# for you, but also simplify event handling with _actions_.  They also
# help you manage resources of inactive states.
class AS::Gamestate
  # During the input phase, _input_ is called on all listeners with
  # the current event.  Add your InputPad and StringInput objects must
  # be added to this array.
  attr_reader :input_listeners
  
  @@current = nil
  
  def initialize
    @redraw = true
    @actions = {}
    @input_listeners = []
  end
  
  # Add an action to run for when the given event occurs.
  def action(event_id, &proc)
    @actions[event_id] = proc
  end
  
  # Method called during the loop to handle events.  Override this
  # method if you wish to handle events in a custom fashion.
  def events
    AS::Event.each(&method(:input))
  end
  
  # Called from AS::Gamestate#events to process individual events.
  # Calls the input method on all objects in the input_listeners
  # array.
  def input(event_id, value)
    proc = @actions[event_id]
    proc.call(value) if proc
    @input_listeners.send_each(:input, event_id, value)
  end
  
  # Called to run the game loop.  Handles events, calls update, and
  # clears and redraws the screen if @redraw is true.
  def game_loop
    return catch(:AS_GAMESTATE_EXIT) do
      loop do
        events
        update
        
        if @redraw
          draw
          AS.swap_buffers
        end
      end
    end
  end
  
  def handle_exception
    print "Exception caught. Attempting to release #{self.class} ... "
    
    begin
      self.release_state if self.respond_to?(:release_state)
    rescue
      puts "failed!"
      puts "Original exception following."
      puts $!
      puts $!.backtrace
      puts
      
      raise
    end
    
    puts "succesful."
    
    raise
  end
  private :handle_exception
  
  # Push the Gamestate onto the state stack.  The previous gamestate
  # has its release_state method called if _releaseOld_ is true.  When
  # the gamestate exits, the previous gamestate has its load_state
  # method called and the argument to AS::Gamestate#stop is returned.
  def run(releaseOld = true)
    @old_state, @@current = @@current, self
    @old_state.release_state if releaseOld and @old_state.respond_to?(:release_state)
    
    self.load_state if self.respond_to?(:load_state)
    begin
      value = game_loop
    rescue
      handle_exception
    end
    self.release_state if self.respond_to?(:release_state)
    
    @@current = @old_state
    @old_state.load_state if @old_state.respond_to?(:load_state)
    
    return value
  end
  
  # Called in the gameloop after events are proccessed.
  def update; end
  
  # Called in the gameloop after calling update and clearing the
  # screen if @redraw is true.
  def draw; end
  
  # Pop the gamestate and return _value_.
  def stop(value = nil)
    AS::Gamestate.stop(value)
  end

  # Pop the current gamestate and return _value_.
  def self.stop(value = nil)
    throw(:AS_GAMESTATE_EXIT, value)
  end
end
