# Utility class to assist in populating VertexBuffers.
class AS::GeometryBuffer
  # A chunk of inserted geometry.
  class Geom
    # The AS::VertexBuffer object that the geom is bound to.
    attr_reader :vbuffer
    # The number of vertexes in the geom.
    attr_reader :count
    # The starting offset of the geom in the buffer.
    attr_accessor :index

    def initialize(vbuffer, index, count)
      @vbuffer = vbuffer
      @index = index
      @count = count
    end
  end

  # The AS::VertexBuffer that the GeometryBuffer is bound to.
  attr_reader :vbuffer

  # _n_vert_ is the number of vertex coordinates. _n_tex_ is the
  # number of texture coordinates, _c_color_ is the number of color
  # coordinates.
  def initialize(n_vert=2, n_tex=0, n_color=0)
    @n_vert = 2
    @n_tex = n_tex
    @n_color = n_color
    
    @stride = @n_vert + @n_tex + @n_color
    
    @geom_hash = {}
    @vbuffer = AS::VertexBuffer.new(@n_vert, @n_tex, @n_color)
  end

  # Repack and upload the buffer.
  def pack_buffer
    index = 0
    
    verts = []
    for (geom, geometry) in @geom_hash do
      geom.index = index
      verts.concat(geometry)
      
      index += geom.count
    end
    
    @vbuffer.upload(verts.pack('f*'))
  end
  
  # Add a new Geom for the given vertex information.
  def add_geom(geometry)
    num = geometry.length
    raise "Incomplete vertexes" unless num%@stride == 0
    
    geom = Geom.new(@vbuffer, nil, num/@stride)
    @geom_hash[geom] = geometry
    pack_buffer
    
    return geom
  end

  # Create a Geom for a texture.
  def add_geom_for_sprite(tex, kx, ky)
    raise "Buffer must be V2T2 format" unless @n_vert == 2 and @n_tex == 2 and @n_color == 0
    
    w, h = [tex.w, tex.h]
    tw, th = (tex.tex_target == GL::TEXTURE_RECTANGLE_EXT) ? [tex.w, tex.h] : [1, 1]
    
    return add_geom([
                        -kx,    -ky,  0,  0,
                     w - kx,    -ky, tw,  0,
                     w - kx, h - ky, tw, th,
                        -kx, h - ky,  0, th,
                    ])
  end
  
  def add_geom_for_tex(tex)
	add_geom_for_sprite(tex, 0, 0)
  end

  # Remove a geom from the buffer.
  def remove_geom(geom)
    @geom_hash.delete(geom)
    pack_buffer
  end
end
