# Convenient event handler.  Once controls are defined, their values
# can be read like a hash.  Controls can be stacked so a joystick axis
# or key pair axis can use the same control name.
class AS::InputPad
  # Set the dead zone for joystick axis used with this pad.
  attr_accessor :dead_zone
  
  # Access the actions hash.
  attr_reader :actions

  def initialize
    reset
    @dead_zone = 0.0
  end
  
  # Remove all defined controls and actions
  def reset
    @controls = {}
    @actions = {}
    @values = {}
  end

  # Retrieve the value of a control.
  def [](name)
    @values[name]
  end

  # Set the value of a control.
  def []=(name, value)
    @values[name] = value
    
    proc = @actions[name]
    proc.call(value) if proc
  end

  # Pass an event.
  def input(event, value)
    proc = @controls[event]
    proc.call(value) if proc
  end
  
  # Add an action to this pad to be invoked when a control is updated.
  def action(name, &proc)
    @actions[name] = proc
  end
  
  # Add a button control to the pad.  Event can be any key or button
  # event.
  def button(name, event, &proc)
    @values[name] = false
    @controls[event] = lambda{|pressed| self[name] = pressed}
    action(name, &proc) if proc
  end

  # Add a button control bound to a key.
  def key(name, key, &proc)
    button(name, AS::Event.for_key(key), &proc)
  end

  # Add a button control bound to a joystick button.
  def jbutton(name, joynum, button, &proc)
    button(name, AS::Event.for_jbutton(joynum, button), &proc)
  end
  
  # Add an axis control bound to a joystick axis.
  def jaxis(name, joynum, axis, reverse=false)
    @values[name] = 0.0
    
    if reverse
      proc = lambda{|value| self[name] = -value}
    else
      proc = lambda{|value| self[name] = value}
    end
    
    @controls[AS::Event.for_jaxis(joynum, axis)] = proc
  end
  
  # Add an axis control bound to a pair of buttons or keys. (digital
  # axis)
  def daxis(name, event1, event2)
    self[name] = 0.0
    
    press1_proc = lambda do|pressed|
      if pressed
        self[name] = -1.0
      elsif self[name] == -1.0
        self[name] = 0.0
      end
    end
    
    press2_proc = lambda do|pressed|
      if pressed
        self[name] = 1.0
      elsif self[name] == 1.0
        self[name] = 0.0
      end
    end
    
    @controls[event1] = press1_proc
    @controls[event2] = press2_proc
  end
  
  # Add an axis control bound to a pair of keys. (key axis)
  def kaxis(name, key1, key2)
    daxis(name, AS::Event.for_key(key1), AS::Event.for_key(key2))
  end
end
