module AS::Scene
  module SceneNode
    def inspect(indent=0)
      str = child.inspect(indent + 1)
      
      "#{"\t"*indent}#{self.class}\n#{str}"
    end
  end
  
  class RenderNode
    def inspect(indent=0)
      "#{"\t"*indent}#{self.class}"
    end
  end
  
  class GroupNode
    def inspect(indent=0)
      arr = children.map{|child| child.inspect(indent + 1)}
      "#{"\t"*indent}AS::Scene::GroupNode\n#{arr.join("\n")}"
    end
    
    # Same as group_node.children << node
    def <<(node)
      self.children << node
    end
    # Same as group_node.children.delete(node)
    def delete(node)
      self.children.delete(node)
    end
  end

  class RenderNode
    # Helper method for creating a RenderNode from a
    # AS::GeometryBuffer::Geom
    def self.new_for_geom(geom, mode)
      new(geom.vbuffer, mode, geom.index, geom.count)
    end
  end

  # Fancy version of a GroupNode that remembers scenegraph structure.
  class Chunk
    # Render root of this ChunkNode.
	attr_accessor :root
	
	# GroupNode used by this ChunkNode
	attr_reader :gnode
  
    # If a block is supplied, a new GroupNode instance is passed to
    # the block and the render root is set to the SceneNode object
    # returned from the block.
    def self.create()
	  chunk = new(GroupNode.new)
      yield(chunk) if block_given?
      
      return chunk
    end

    # _gnode_ should be a GroupNode to use when adding
    # children. _root_ should be the SceneNode object that
    # SceneNode#render is called on to render the ChunkNode.
    def initialize(gnode=AS::Scene::GroupNode.new, root=nil)
      @gnode = gnode
      @root = (root or gnode)
      @child_chunks = {}
    end

    # Find the named ChunkNode.
    def [](index)
      @child_chunks[index]
    end
	
	def []=(index, chunk)
		add_chunk(index, chunk)
	end

    # Add a child ChunkNode.
    def add_chunk(index, chunk)
      children << chunk.root
      @child_chunks[index] = chunk
    end

    # Remove a ChunkNode.
    def delete_chunk(index)
      chunk = @child_chunks.delete(index)
      children.delete(index.root)
      
      return chunk
    end
    
    # Calls #children on the underlying GroupNode.        
    def children
      @gnode.children
    end

    # Calls #<< on the underlying GroupNode.
    def <<(node)
      @gnode << node
    end

    # Calls #delete on the underlying GroupNode.
    def delete(node)
      children.delete(node)
    end

    # Calls #render on the underlying root SceneNode.
    def render
      @root.render
    end
    
    def inspect
      @root.inspect
    end
  end
end
