require 'base/graph.rb'
require 'base/gameworld.rb'

require 'game/gamepiece.rb'

class GameBoard < GameWorld
	W = 100
	H = 75
	
	attr_reader :score, :score_string
	
	def initialize(gamestate)
    super()
    @gamestate = gamestate
    
    @pieces = []
		@pairs = []
		@wildcard_pairs = []
		
		set_up_space
	end
	
	def create_view
		AS::Scene::Chunk.create{|chunk|
			chunk[:balls] = AS::Scene::Chunk.new
		}
	end
	
	def remove_piece(piece)
		@pieces.delete(remove_game_object(piece))
	end
	
	def add_starting_pieces(num)
		num.times do
			radius = NormalPiece.rand_radius
			x = radius + rand(W - 2*radius)
			y = radius + rand(H - 2*radius)
			
			@pieces << add_game_object(NormalPiece.new(vec2(x,y), radius))
		end
	end
	
	def set_up_space
		space.gravity = vec2(0, -100)

		inf = 1.0/0.0
		static_body = CP::Body.new(inf, inf)
		
		seg = CP::Shape::Segment.new(static_body, vec2(0, 2*H), vec2(0, 0), 0)
		seg.u = 1.0
		space.add_static_shape(seg)
		
		seg = CP::Shape::Segment.new(static_body, vec2(0, 0), vec2(W, 0), 0)
		seg.u = 1.0
		space.add_static_shape(seg)
		
		seg = CP::Shape::Segment.new(static_body, vec2(0, 2*H), vec2(W, 2*H), 0)
		seg.u = 1.0
		space.add_static_shape(seg)
		
		seg = CP::Shape::Segment.new(static_body, vec2(W, 0), vec2(W, 2*H), 0)
		seg.u = 1.0
		space.add_static_shape(seg)
		
		add_collision_funcs
	end
	
	def add_collision_funcs
		#This block of code will be executed when the collisions specified below occur
	
		GamePiece::Colors.each do|color|
			space.add_collision_func(color, color){|a, b| @pairs << [a.piece, b.piece]}
		end

		block = lambda{|a, b| @wildcard_pairs << [a.piece, b.piece]}

		#Wildcards should match themselves and every other color.		
		#Put them in a special list for removal if one is clicked on!
		space.add_collision_func(WildCardPiece::Color, WildCardPiece::Color, &block)
		GamePiece::Colors.each do|color|
			space.add_collision_func(color, WildCardPiece::Color, &block)
		end
	end
	
	FillVelocity = 30
	def add_piece(pieceClass)
		radius = GamePiece.rand_radius #should be pieceClass.rand_radius
 		pos = vec2(radius + rand(W - 2*radius), 1.5*H + radius)
		
		piece = pieceClass.new(pos, radius)
		piece.body.v = vec2((2*rand - 1)*FillVelocity, 0)

		@pieces << add_game_object(piece)
	end
	
	def piece_at(pos)
		@pieces.find{|piece| piece.selected?(pos)}
	end
	
	def pieces_near(pos, radius)
		@pieces.find_all{|piece| piece.selected?(pos, radius)}
	end
	
	# Number of pieces on the board.
	def count
		@pieces.length
	end
	
	def substeps
		2
	end
	
	# This piece should be removed, as it has been matched out.
	# It was in a group of like-colored pieces, and is thus being removed.
	def match_out_piece(piece)
		remove_piece(piece)
		piece.matched_out(self)
	end
	
	# called by GamePiece subclasses for custom actions that require scoring
	# ex: BombPiece#matched_out
	def group_cleared(group)
		@gamestate.group_cleared(group)
	end
	
	def reset_pairs
		@pairs.clear
		@wildcard_pairs.clear	
	end
	
	def match_wildcard(wildcard)
		puts 'yo'
		matched_pairs = @wildcard_pairs.select{|a, b| a == wildcard or b == wildcard}
		
		graph = Graph.new(@objects, matched_pairs + @pairs)
		island = graph.flood(wildcard)
		clear_group(island)
	end
	
	def clear_group(group)
		group_cleared(group)
		group.each{|piece| match_out_piece(piece)}
	end
	
	def update
		reset_pairs
		super
		    		
		graph = Graph.new(@objects, @pairs)
		islands = graph.islands.select{|island| island.length >= 4}
		islands.each{|island| clear_group(island)}
		
		add_pieces_on_timer
		@gamestate.game_over if count > 120
	end
	
	def rate(i)
		(120*(1.15**-i)).to_i
	end
	
	PieceClasses = [
		BombPowerup,
		WildCardPiece,
	]

	# Add pieces to the gameboard occasionally. I suspect
	# we might want to pace the game here based on the player's score
	# although, pacing by the amount of time they've been playing might be better.
	def add_pieces_on_timer
		rate = rate(@gamestate.timer.ticks/1800)
		#puts "#{@gamestate.timer.ticks/3600} - #{rate}"

		if @gamestate.timer.ticks%rate == 0 
			if rand < 0.20
				add_piece(PieceClasses[rand(PieceClasses.length)]) 
			else
				add_piece(NormalPiece)
			end
		end
	end
	
	
	def draw
		super
		
		GL.Color3f(1.0, 1.0, 1.0)
		GL.Disable(GL::BLEND)
		GL.Disable(GL::TEXTURE_2D)
		GL.Disable(GL::TEXTURE_RECTANGLE_EXT)
#		Sparks.draw(@pairs.map{|a, b| [a.body.p, b.body.p]});
	end
end
