GBuff = AS::GeometryBuffer.new(2, 2, 0)

verts = [
	-1, -1, 0,  0,
	-1, 1, 0,  1,
	1, 1, 1, 1,
	1, -1, 1, 0,
]

Geom = GBuff.add_geom(verts)

class GamePiece

	Colors = [
		AS::Color::Red.pack,
		AS::Color::Green.pack,
		AS::Color::Blue.pack,
		AS::Color::Yellow.pack,
		AS::Color::Cyan.pack,
		AS::Color::Magenta.pack,
	]

	def self.rand_radius
		3 + rand*2
	end

	class Shape < CP::Shape::Circle
		attr_reader :piece
	
		def initialize(piece, type, body, radius)
			super(body, radius, vec2(0,0))
			
			self.u = 0.4
			self.collision_type = type
			
			@piece = piece
		end
	end
	
	class View
		
		Scale = 256.0/240.0
		
		def initialize(body, radius, end_of_node)
			@ball = AS::Scene::BodyNode.new(body){
				scale = radius*Scale
				matrix = AS::PackedMatrix.new(AS::Transform2D.scale(scale, scale))
				
			 	AS::Scene::MatrixNode.new(GL::MODELVIEW, matrix, end_of_node)
			}
		end
		
		def add_to_view(chunk)
			chunk << @ball
		end
		
		def remove_from_view(chunk)
			chunk.delete(@ball)
		end
	end

	attr_reader :body
	
	def initialize(pos, radius, view_node, collision_type)
		@radius = radius
		mass = 2*Math::PI*@radius**2/100
		moment = CP.moment_for_circle(mass, @radius, 0, vec2(0,0))
	
		@body = CP::Body.new(mass, moment)
		@body.p = pos
			
		@shape = Shape.new(self, collision_type, @body, @radius)
		@node = View.new(@body, @radius, view_node)	
	end
	
	def add_to_world(world)
		space = world.space
		chunk = world.view_root
		
		space.add_body(@body)
		space.add_shape(@shape)
		
		@node.add_to_view(chunk[:balls])
	end
	
	def remove_from_world(world)
		space = world.space
		chunk = world.view_root
	
		space.remove_body(@body)
		space.remove_shape(@shape)
		
		@node.remove_from_view(chunk[:balls])
	end
	
	def matched_out(board)
		#do nothing
	end
	
	def clicked_on(board)
		#do nothing
	end
	
	def selected?(pos, radius=0)
		@body.p.near?(pos, @radius + radius)
	end
end

class NormalPiece < GamePiece
	def initialize(body, radius)
		color = Colors[rand(Colors.length)]
		super(body, radius, finish_view_object(color), color)
	end
	
	def clicked_on(board)
		ClickSound.play
		4.times{board.add_piece(NormalPiece)}
	end
	
	def matched_out(board)
		MatchBombSound.play
	end
	
	def finish_view_object(color)
		AS::Scene::TextureNode.new(BallTex){
			AS::Scene::ColorNode.new(color){
				AS::Scene::RenderNode.new_for_geom(Geom, GL::QUADS)
			}
		}
	end
end

class WildCardPiece < GamePiece
	Color = AS::Color::White.pack

	def initialize(body, radius)
		super(body, radius, finish_view_object(Color), Color)
	end
	
	def clicked_on(board)
		BombSound.play
		board.match_wildcard(self)
		4.times{board.add_piece(NormalPiece)}
	end
	
	def matched_out(board)
		MatchBombSound.play
	end
	
	def finish_view_object(color)
		AS::Scene::TextureNode.new(BallTex){
			AS::Scene::ColorNode.new(color){
				AS::Scene::RenderNode.new_for_geom(Geom, GL::QUADS)
			}
		}
	end
end


class BombPowerup < GamePiece
	def initialize(body, radius)
		color = Colors[rand(Colors.length)]
		super(body, radius, finish_view_object(color), color)
	end
	
	def finish_view_object(color)
		insetTex = AS::Scene::TextureNode.new(BombTex){
			AS::Scene::BlendNode.new(nil, GL::DST_COLOR, GL::ZERO){
				AS::Scene::RenderNode.new_for_geom(Geom, GL::QUADS)
			}
		}
		mainTex = AS::Scene::TextureNode.new(BallTex){
			AS::Scene::ColorNode.new(color){
				AS::Scene::RenderNode.new_for_geom(Geom, GL::QUADS)
			}
		}
		return AS::Scene::GroupNode.new([mainTex, insetTex])
	end
	
	def matched_out(board)
		MatchBombSound.play
		radius = 15
		v_mag = 60
		
		pieces = board.pieces_near(self.body.p, radius)
		
		board.group_cleared(pieces)
		
		pieces.each{|piece| 
			board.remove_piece(piece)
		}
	end
	
	def clicked_on(board)
		super
		BombSound.play
		radius = 30
		v_mag = 60
		
		pieces = board.pieces_near(self.body.p, radius)
		#pieces.each{|piece| board.remove_piece(piece)}
		pieces.each do|piece|
			continue if piece == self
			
			delta = piece.body.p - self.body.p
			delta_len = delta.length
			mag = (radius - delta_len)/(delta_len**2)*v_mag
			
			piece.body.v += delta*mag
		end
	end
end