require 'game/gameboard.rb'
require 'game/score.rb'

require 'effects/sparks.rb'
require 'effects/scorefloaty.rb'

# Author::    Scott Lembcke and Andy Korth
# Copyright:: Copyright (c) 2007 Howling Wolf Software
#
# This Gamestate class is responsible for keeping track of the board,
# the effects that might be displayed on the screen, the HUD, the player's
# score, and handling input when the game is being played.
class Gamestate < AS::Gamestate
	W, H = 100, 75
	
	BallRect = [105, 74, 604, 406]
	
	ScoreString = "Score: %s"
	
	attr_reader :timer, :score

	def initialize
		super
		setup_actions
		
		@score = Score.new
		@score_string = sprintf(ScoreString, 0)
		@ball_count_string = ""
		
		@board = GameBoard.new(self)
		@board.add_starting_pieces(STARTINGPIECES)
		
		make_scenegraph
				
		@timer = AS::TickTimer.new(FRAMERATE)
		#@fps = FPSDisplay.new
	end

	SCORE_FONT_SCALE = 0.05
	
	def make_scenegraph
		@effects = AS::Scene::Chunk.create{|chunk|
			chunk[:floaties] = ScoreFloatyChunk.new
		}

		#The view root contains the scenegraph nodes for the hud
		@view_root = AS::Scene::Chunk.create{|chunk|
			chunk.root = PreMultAlphaBlendNode.new(chunk.root)
		
			chunk[:hud] = AS::Scene::Chunk.create{|chunk|
				chunk.root = PreMultAlphaBlendNode.new(chunk.root)
			
				matrix = AS::Transform2D.translate(5, 63).scale(SCORE_FONT_SCALE, SCORE_FONT_SCALE)
				packed_matrix = AS::PackedMatrix.new(matrix)
				chunk << AS::Scene::MatrixNode.new(GL::MODELVIEW, packed_matrix){
					TextNode.new(ThisFont, @score_string)
				}

				matrix = AS::Transform2D.translate(5, 58).scale(SCORE_FONT_SCALE, SCORE_FONT_SCALE)
				packed_matrix = AS::PackedMatrix.new(matrix)
				chunk << AS::Scene::MatrixNode.new(GL::MODELVIEW, packed_matrix){
					TextNode.new(ThisFont, @ball_count_string)
				}
			}
		}
		
		#contains nodes for playing area, like the balls and other clicky things. It's inset.
		@playing_area_root = AS::Scene::Chunk.create{|chunk|
			chunk.root = PreMultAlphaBlendNode.new(chunk.root)

			chunk[:board] = @board.view_root
			chunk[:effects] = @effects	
		}
		
	end
	
	def setup_actions
		action(AS::Event.for_mbutton(1)){|state| click if state}
		action(AS::Event.for_mbutton(3)){|state| @board.add_piece(NormalPiece) if state}
		action(AS::Event.for_mbutton(2)){|state| @board.add_piece(BombPowerup) if state}
		action(AS::Event.for_key(:Key_ESCAPE)){stop}
	end
	
	def group_cleared(group)
		num = group.length
	
		group_score = @score.group_cleared(group.length, @timer.ticks)
		@score_string.replace(sprintf(ScoreString, commas(@score.score)))
		
		sum = group.inject(vec2(0,0)){|sum, piece| sum += piece.body.p}
		@effects[:floaties].add_floaty(sum/num, group_score)
	end
	
	def click
		bx, by, bw, bh = *BallRect
	
		mx, my = *AS::Event.mouse_pos
		x = Float(mx - bx)/bw*GameBoard::W
		y = Float(my - by)/bh*GameBoard::H
		pos = vec2(x, y)
		
		piece = @board.piece_at(pos)
		if @board.remove_piece(piece)
			piece.clicked_on(@board)
		end
	end
	
	def update
		@redraw = @timer.sleep

		@board.update
		@ball_count_string.replace("Balls: #{@board.count}/120")
		
		@effects[:floaties].update
	end
	
	def game_over
		puts "time: #{@timer.ticks/60} seconds, score: #{score.score}"
		stop
	end
	
	def draw
		GL.Viewport(0, 0, 800, 600)
		#AS.clear_screen
		
		GL.Disable(GL::TEXTURE_2D)
		GL.Disable(GL::TEXTURE_RECTANGLE_EXT)
		GL.Disable(GL::BLEND)
		GL.Color3f(1,1,1)
		GL.Enable(BGTex.tex_target)
		BGTex.bind
		BGTex.copy_rect(0, 0, 800, 600, 0, 0, 100, 75)

		@view_root.render

		GL.Viewport(*BallRect)
		@playing_area_root.render

		#@fps.draw
		AS.print_gl_errors
	end
end


#class FPSDisplay
#	def initialize(ticks=60)
#		@fps = AS::FPSCounter.new(FRAMERATE)
#		@fps_string = ""
#		
#		scale = 0.05
#		matrix = AS::Transform2D.translate(3, 3).scale(scale, scale)
#		packed_matrix = AS::PackedMatrix.new(matrix)
#		@view = AS::Scene::MatrixNode.new(GL::MODELVIEW, packed_matrix){
#			AS::Scene::BlendNode.new(nil, GL::ONE, GL::ONE_MINUS_SRC_ALPHA){
#				TextNode.new(ThisFont, @fps_string)
#			}
#		}
#	end
#	
#	def draw
#		@fps_string.replace(sprintf("fps: %.2f", @fps.current)) if @fps.tick
#		@view.render
#	end
#end